import { NodePart, nothing } from './lit-html.js';
export const detachNodePart = (part) => {
    const fragment = document.createDocumentFragment();
    const state = {
        __value: part._value,
        __fragment: fragment,
    };
    let start = part._startNode.nextSibling;
    let nextNode;
    while (start !== part._endNode) {
        nextNode = start.nextSibling;
        fragment.append(start);
        start = nextNode;
    }
    part._value = nothing;
    return state;
};
export const restoreNodePart = (part, state) => {
    // TODO (justinfagnani): make an interal-only interface
    part._commitNode(state.__fragment);
    part._value = state.__value;
};
const createMarker = () => document.createComment('');
export const createAndInsertPart = (containerPart, refPart) => {
    const container = containerPart._startNode.parentNode;
    const refNode = refPart === undefined ? containerPart._endNode : refPart._startNode;
    const startNode = container.insertBefore(createMarker(), refNode);
    const endNode = container.insertBefore(createMarker(), refNode);
    return new NodePart(startNode, endNode, containerPart.options);
};
export const setPartValue = (part, value) => {
    part._setValue(value);
    return part;
};
export const getPartValue = (part) => part._value;
export const insertPartBefore = (containerPart, part, refPart) => {
    const container = containerPart._startNode.parentNode;
    const refNode = refPart ? refPart._startNode : containerPart._endNode;
    const endNode = part._endNode.nextSibling;
    if (endNode !== refNode) {
        reparentNodes(container, part._startNode, endNode, refNode);
    }
};
export const removePart = (part) => {
    removeNodes(part._startNode, part._endNode.nextSibling);
};
/**
 * Reparents nodes, starting from `start` (inclusive) to `end` (exclusive),
 * into another container (could be the same container), before `before`. If
 * `before` is null, it appends the nodes to the container.
 */
const reparentNodes = (container, start, end = null, before = null) => {
    while (start !== end) {
        const n = start.nextSibling;
        container.insertBefore(start, before);
        start = n;
    }
};
/**
 * Removes nodes, starting from `start` (inclusive) to `end` (exclusive), from
 * `container`.
 */
const removeNodes = (start, end = null) => {
    while (start !== end) {
        const n = start.nextSibling;
        start.remove();
        start = n;
    }
};
//# sourceMappingURL=parts.js.map